// camera.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CCamera::CCamera(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
CCamera::~CCamera(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CCamera::Loop(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CCamera::Init(void)
// ------------------------------------------------------------------------------------------------
{
	if (!CreateCameraAndLight()) return(false);
	return(true);
}

// ------------------------------------------------------------------------------------------------
void CCamera::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CCamera::Reset(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CCamera::CreateCameraAndLight(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	gl_Data.pCamera = gl_Data.pSceneManager->addCameraSceneNode(
												gl_Data.pSceneManager->getRootSceneNode(),
												irr::core::vector3df(0.0f, 7.0f, -13.0f),   // position
												irr::core::vector3df(0.0f, -0.5f, 0.0f), // lookat point
												0); // Need 0 instead of standard -1, as it would be
												    // found with the ray hit test bitmask otherwise
												    // (-1 = ffff), running into every hit test
	if (!gl_Data.pCamera) return(false);

	gl_Data.pCamera->setFOV(90.0f*PI/180.0f); // Overriding bad standard value (which is 45deg) with 90deg
	
    // Create the point light as a child node of the camera. So, it will always follow.
	// Position is a bit behind and up the camera
	gl_Data.pLight = gl_Data.pSceneManager->addLightSceneNode( gl_Data.pCamera,
															   irr::core::vector3df(0.0f, 1.0f, -1.0f),
															   irr::video::SColor(2,2,2,2),
															   (irr::s32) 2000);  // Radius

	return(true);
}

// ------------------------------------------------------------------------------------------------
void CCamera::SetCameraPosition(irr::core::vector3df vPos)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	gl_Data.pCamera->setPosition(vPos);
	gl_Data.pCamera->updateAbsolutePosition();

	gl_Data.pCamera->setTarget(irr::core::vector3df(0.0f,-0.5f,0.0f));
	
	gl_Data.pCamera->updateAbsolutePosition();
	gl_Data.pLight->updateAbsolutePosition();
}

// ------------------------------------------------------------------------------------------------
void CCamera::MoveCameraLeftRight(irr::s32 iPosDelta) // 1 is left turn, -1 = right
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	static irr::s32 iOld = 0;

	// Check if animation still running: left/right
	irr::core::vector3df vPos = gl_Data.pCamera->getAbsolutePosition();
	
	// Return if cam is not at one of its home positions
	if ((vPos.X != 0.0f) && (vPos.X != 13.0f) && (vPos.X != -13.0f)) return;
	if ((vPos.Z != 0.0f) && (vPos.Z != 13.0f) && (vPos.Z != -13.0f)) return;

	// Check if animation still running: up/down
	if (vPos.Y != floor(vPos.Y)) return;
	
	irr::s32 iNew = iOld + iPosDelta;
	if (iNew <0) iNew = 3;
	else if (iNew > 3) iNew = 0;	
	
	irr::core::vector3df newFixedPos;
	switch (iNew)
	{
	case 0: newFixedPos = irr::core::vector3df(  0.0f, vPos.Y, -13.0f); break;
	case 1: newFixedPos = irr::core::vector3df(-13.0f, vPos.Y,   0.0f); break;
	case 2: newFixedPos = irr::core::vector3df(  0.0f, vPos.Y, +13.0f); break;
	case 3: newFixedPos = irr::core::vector3df(+13.0f, vPos.Y,   0.0f); break;
	}	
	
	gl_Data.pCamera->removeAnimators();
	iOld = iNew;	

	irr::scene::ISceneNodeAnimator* pA = gl_Data.pSceneManager->createFlyStraightAnimator(vPos,
																						  newFixedPos,
																						  3000);
	gl_Data.pCamera->addAnimator(pA);
	pA->drop();
}

// ------------------------------------------------------------------------------------------------
void CCamera::MoveCameraUpDown(irr::s32 iPosDelta) // 1 is up, -1 = down
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// Check if animation still running: left/right
	irr::core::vector3df vPos = gl_Data.pCamera->getAbsolutePosition();
	
	// Return if cam is not at one of its home positions
	if ((vPos.X != 0.0f) && (vPos.X != 13.0f) && (vPos.X != -13.0f)) return;
	if ((vPos.Z != 0.0f) && (vPos.Z != 13.0f) && (vPos.Z != -13.0f)) return;

	// Check if animation still running: up/down
	if (vPos.Y != floor(vPos.Y)) return;

	irr::f32 fNew = vPos.Y + iPosDelta;
	if (fNew <1) fNew = 1;
	else if (fNew > 13) fNew = 13;
	
	irr::core::vector3df newFixedPos(vPos.X, fNew, vPos.Z);
	
	gl_Data.pCamera->removeAnimators();

	irr::scene::ISceneNodeAnimator* pA = gl_Data.pSceneManager->createFlyStraightAnimator(vPos,
																						  newFixedPos,
																						  500);
	gl_Data.pCamera->addAnimator(pA);	
	pA->drop();

}


