// debug.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CDebug::CDebug(void)
// ------------------------------------------------------------------------------------------------
{
	m_pDebugWindow = NULL;
	m_pDebugText   = NULL;
}

// ------------------------------------------------------------------------------------------------
CDebug::~CDebug(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CDebug::Loop(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	if (!m_pDebugWindow) return;

	// Update the data
	
	irr::core::vector3df vCamPos = gl_Data.pCamera->getAbsolutePosition();
	irr::core::vector3df vCamRot = gl_Data.pCamera->getRotation();
	irr::core::vector3df vDicePos = gl_Data.Dice.m_pDiceNode->getAbsolutePosition();
	irr::core::stringw sState;
	irr::core::stringw sIndices = L"Rel-Indices\n";

	for (irr::s32 i=0;i<4;i++)
	{
		wchar_t c[200];
		swprintf(c,100,L"P: %i,%i,%i,%i\n", gl_Data.Map.m_Tokens[i][0].iRelativeIndex,
											gl_Data.Map.m_Tokens[i][1].iRelativeIndex,
											gl_Data.Map.m_Tokens[i][2].iRelativeIndex,
											gl_Data.Map.m_Tokens[i][3].iRelativeIndex);
		sIndices.append(c);
	}
	
	switch (gl_Data.GameState)
	{
	case autogame:   sState = L"auto";   break;
	case idle:       sState = L"idle";   break;
	default:         sState = L"UNDEFINED";break;
	}	
	
	static wchar_t dummy[1024]; 
	swprintf(dummy,1000,L"%ix%i@%i\nFPS: %i\nCamPos: %.1f %.1f %.1f\nCamRot: %.1f, %.1f, %.1f\nPrimitives: %i\nMeshCache: %i\nPaused: %i\nState: %s\n%sActive: %i\nDicePos: %.1f, %.1f, %.1f",
												gl_Data.ScreenXY.Width,
												gl_Data.ScreenXY.Height,
												gl_Data.uColorDepth,
												gl_Data.pVideoDriver->getFPS(),
												vCamPos.X, vCamPos.Y, vCamPos.Z,
												vCamRot.X, vCamRot.Y, vCamRot.Z,
												gl_Data.pVideoDriver->getPrimitiveCountDrawn(),
												gl_Data.pSceneManager->getMeshCache()->getMeshCount(),
												gl_Data.bGamePaused,
												sState.c_str(),
												sIndices.c_str(),
												gl_Data.iActivePlayer,
												vDicePos.X, vDicePos.Y, vDicePos.Z);	
	
	m_pDebugText->setText(dummy);
}

// ------------------------------------------------------------------------------------------------
bool CDebug::Init(void)
// ------------------------------------------------------------------------------------------------
{
	return(true);
}

// ------------------------------------------------------------------------------------------------
void CDebug::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CDebug::Reset(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CDebug::ToggleDebugWindow(void)
// ------------------------------------------------------------------------------------------------
{
	if (!m_pDebugWindow) CreateDebugWindow();
	else DeleteDebugWindow();
}

// ------------------------------------------------------------------------------------------------
void CDebug::CreateDebugWindow(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	irr::core::rect<irr::s32> rec(20,20,220,320);	
	m_pDebugWindow = gl_Data.pGUIEnv->addWindow(rec,false,L"Debug Data",gl_Data.pGUIEnv->getRootGUIElement());
	
	irr::gui::IGUIButton* pB = m_pDebugWindow->getCloseButton();
	pB->setVisible(false);

	rec = m_pDebugWindow->getRelativePosition();
	rec.LowerRightCorner.X -= 5;
	rec.LowerRightCorner.Y -= 5;
	rec.UpperLeftCorner.X =  5;
	rec.UpperLeftCorner.Y = 20;

	m_pDebugText = gl_Data.pGUIEnv->addStaticText(L"",rec,false,true,m_pDebugWindow);
}

// ------------------------------------------------------------------------------------------------
void CDebug::DeleteDebugWindow(void)
// ------------------------------------------------------------------------------------------------
{
	SAFEREMOVE(m_pDebugWindow);
}

