// dice.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CDice::CDice(void)
// ------------------------------------------------------------------------------------------------
{
	m_pDiceNode = NULL;
}

// ------------------------------------------------------------------------------------------------
CDice::~CDice(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CDice::Loop(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CDice::Init(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_dice.x");
	if (!pM) return(false);

	m_pDiceNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),-1,irr::core::vector3df(20.0f,0.0f,0.0f));
	m_pDiceNode->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));

	m_pDiceNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
	m_pDiceNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true);
		
	ShowDice(false);
		
	return(true);
}

// ------------------------------------------------------------------------------------------------
void CDice::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CDice::Reset(void)
// ------------------------------------------------------------------------------------------------
{
	ShowDice(false);
}

// ------------------------------------------------------------------------------------------------
irr::s32 CDice::Roll(irr::s32 iPlayer)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	irr::s32 iResult;
	
	irr::s32 iT = (rand())%1000+5; 
	for (irr::s32 i=0;i<iT;i++) iResult = rand()%6+1; // 1..6

	// Check if player got all items out of game. Then, force 6
	irr::s32 dummy;
	if (!gl_Data.Map.CheckIfPlayerGotIngameItems(iPlayer, dummy)) iResult=6;

	ShowDice(true);
	InitiateDiceRoll(iPlayer, iResult);

	return(iResult);
}

// ------------------------------------------------------------------------------------------------
void CDice::InitiateDiceRoll(irr::s32 iPlayer, irr::s32 iResult)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	irr::core::vector3df vDPos;	
	switch (iPlayer)
	{
	case 0: vDPos = irr::core::vector3df(-3.0f, 10.0f, -3.0f); break;
	case 1: vDPos = irr::core::vector3df(-3.0f, 10.0f,  3.0f); break;
	case 2: vDPos = irr::core::vector3df( 3.0f, 10.0f,  3.0f); break;
	case 3: vDPos = irr::core::vector3df( 3.0f, 10.0f, -3.0f); break;
	}
	
	m_pDiceNode->setPosition(vDPos);
	
	irr::core::vector3df vRot;
	switch (iResult)
	{
		case 1: vRot.set(-135.0f,  0.0f, -90.0f); break;
		case 2: vRot.set(  90.0f, 45.0f,   0.0f); break;
		case 3: vRot.set(   0.0f, 45.0f,   0.0f); break;
		case 4: vRot.set( 180.0f, 45.0f,   0.0f); break;
		case 5: vRot.set( -90.0f, 45.0f,   0.0f); break;
		case 6: vRot.set(  45.0f,  0.0f,  90.0f); break;
	}
	m_pDiceNode->setRotation(vRot);

	irr::core::vector3df gravity(0.0f,-0.03f,0.0f);
	irr::core::vector3df radius(0.4f,0.4f,0.4f);

	m_pDiceNode->removeAnimators();
	
	irr::scene::ISceneNodeAnimatorCollisionResponse* r = 
		gl_Data.pSceneManager->createCollisionResponseAnimator(gl_Data.Map.m_pPlaneNode->getTriangleSelector(),
															   m_pDiceNode,
															   radius,
															   gravity);
	
	m_pDiceNode->addAnimator(r);
	r->drop();
}


// ------------------------------------------------------------------------------------------------
void CDice::ShowDice(bool bShowIt)
// ------------------------------------------------------------------------------------------------
{
	m_pDiceNode->setVisible(bShowIt);
}

// ------------------------------------------------------------------------------------------------
bool CDice::IsDiceOnTheMove(void)
// ------------------------------------------------------------------------------------------------
{
	if (m_pDiceNode->getAbsolutePosition().Y < 0.5f) return(false);
	else return(true);
}
