// global.h

#define MY_PLATFORM_WIN32 1
#define MY_PLATFORM_LINUX 2

// ====================================================
// Change here to compile for another platform

#define PLATFORM MY_PLATFORM_LINUX
//#define PLATFORM MY_PLATFORM_WIN32

// ====================================================

#if PLATFORM == MY_PLATFORM_WIN32
	#include <Windows.h>
	#include <time.h>
	// You would need to enter your own path here
        #include "./include/IMeshCache.h"
	#include "./include/irrlicht.h"
	#pragma comment(lib, "./lib/Irrlicht.lib")
#else
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <time.h>
	#include <IMeshCache.h>
	#include <irrlicht.h>
#endif

// ====================================================
// My definitions. 

#define SAFERELEASE(p)      if (p) {(p)->Release(); (p) = NULL;}
#define SAFEDELETE(p)       if (p) {delete(p);      (p) = NULL;}
#define SAFEDELETE_ARRAY(p) if (p) {delete[](p);    (p) = NULL;}
#define SAFEREMOVE(p)       if (p) {(p)->remove();  (p) = NULL;}
#define SAFEDROP(p)         if (p) {(p)->drop();    (p) = NULL;}

#define MIN(a, b) ((a) < (b) ? (a) : (b))						
#define MAX(a, b) ((a) > (b) ? (a) : (b))

#define PI 3.1415926535f

#define DEGTORAD(x) ((x) * PI / 180.0f)
#define RADTODEG(x) ((x) * 180.0f / PI)

// ====================================================
// Global IDs 

#define ID_BUTTON_CAMLEFT 1
#define ID_BUTTON_CAMRIGHT 2
#define ID_BUTTON_CAMUP 3
#define ID_BUTTON_CAMDOWN 4

#define ID_BUTTON_GAMEAUTO 5
#define ID_BUTTON_GAMEQUIT 7

#define ID_BUTTON_INGAMEPAUSE 8
#define ID_BUTTON_INGAMEABORT 9

// ====================================================
// The fields tokens walk on 
typedef struct 
{
	irr::core::vector3df vPosition;
	irr::scene::ISceneNode* pNode;

} ONEFIELD;

// ====================================================
// The players' tokens 
typedef struct 
{
	irr::s32 iAbsoluteIndex; // index 0 is the init field of player1
	irr::s32 iRelativeIndex; // index 0 is the init field of this very player
	irr::scene::ISceneNode* pNode;

} ONETOKEN;

// ====================================================
// Used by the "movejob engine". Partly, straight movement bit 
typedef struct  
{
	irr::scene::ISceneNode* pNode;
	irr::core::vector3df vEndPosition;

} MOVEJOB;

// ====================================================
// The game state 

typedef enum E_GAME_STATE
{
  idle = 0,
  autogame,
};

// ====================================================
// The player state 
enum PLAYERSTATE
{
	PS_NONE, PS_ACTIVE, PS_WAIT1, PS_ROLL, PS_MOVE, PS_WAIT2
};

// ====================================================
// Global includes

#include "CGUIexScrollWindow.h"
#include "map.h"
#include "camera.h"
#include "debug.h"
#include "input.h"
#include "gui.h"
#include "player.h"
#include "dice.h"

// ====================================================
// The global data structure 

typedef struct 
{
	irr::IrrlichtDevice*		  pIrrlichtDevice;
	irr::video::IVideoDriver*	  pVideoDriver;
	irr::scene::ISceneManager*	  pSceneManager;
	irr::gui::IGUIEnvironment*	  pGUIEnv;
	irr::ILogger*				  pLogger;

	irr::scene::ILightSceneNode*  pLight;
	irr::scene::ICameraSceneNode* pCamera;
	
	E_GAME_STATE GameState;
	bool bGamePaused;
	irr::s32 iActivePlayer;

	irr::core::dimension2d<irr::s32> ScreenXY;
	irr::u32 uColorDepth;

	CCamera Camera;
	CMap	Map;
	CGui    Gui;
	CInput  Input;
	CDebug  Debug;
	CPlayer Player[4];
	CDice   Dice;

} GLOBALDATA;

// ====================================================
// Debug Helpers

inline void DB(char* pInfo, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) /// Inline function for easy debug output (string)
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
    sprintf(buffer,"%s",pInfo);
    gl_Data.pLogger->log(buffer, lvl);
}

inline void DB(char* pInfo, float fVal, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) /// Inline function for easy debug output (string+float)
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
    sprintf(buffer,"%s %f",pInfo,fVal);
    gl_Data.pLogger->log(buffer,lvl);
}

inline void DB(char* pInfo, irr::s32 iVal, irr::ELOG_LEVEL lvl = irr::ELL_INFORMATION) /// Inline function for easy debug output (string+irr::s32)
{
    extern GLOBALDATA gl_Data;
	char buffer[2048];
    sprintf(buffer,"%s %i",pInfo,iVal);
    gl_Data.pLogger->log(buffer, lvl);
}

// ====================================================
// Global functions available in main.cpp

bool gl_Init(void);
bool gl_InitObjects(void);
bool gl_InitScreen(void);
void gl_Reset(void);
void gl_Exit(void);
void gl_Loop(void);
void gl_SetGameState(E_GAME_STATE newState);

// ====================================================
