// gui.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CGui::CGui(void)
// ------------------------------------------------------------------------------------------------
{
	m_pHelpTextBox = NULL;
	m_pWinWindow   = NULL;
}

// ------------------------------------------------------------------------------------------------
CGui::~CGui(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CGui::Loop(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CGui::Init(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// for the ease of use
	gl_Data.pGUIEnv = gl_Data.pIrrlichtDevice->getGUIEnvironment();
	
	// Load our own font
	m_pFont = gl_Data.pGUIEnv->getFont("./data/CM_Arial_12.bmp");
	if (!m_pFont) return(false);
	gl_Data.pGUIEnv->getSkin()->setFont(m_pFont);

	// Changing alpha of windows etc. First, upper right color, then, lower left
	irr::video::SColor col = gl_Data.pGUIEnv->getSkin()->getColor(irr::gui::EGDC_3D_FACE);
	col.setAlpha(230);
	gl_Data.pGUIEnv->getSkin()->setColor(irr::gui::EGDC_3D_FACE, col);

	col = gl_Data.pGUIEnv->getSkin()->getColor(irr::gui::EGDC_3D_SHADOW );
	col.setAlpha(230);
	gl_Data.pGUIEnv->getSkin()->setColor(irr::gui::EGDC_3D_SHADOW , col);	 

	// Other initialisations
	if (!InitCamButtons()) return(false);
	if (!InitGameButtons()) return(false);
	if (!InitInGameButtons()) return(false);
	if (!InitDemoSign()) return(false);

	Reset();
	
	return(true);
}

// ------------------------------------------------------------------------------------------------
void CGui::Reset(void)
// ------------------------------------------------------------------------------------------------
{
	SetPauseSign(false);
	SetRunningGame(false);
	ShowWinner(0, false);
}

// ------------------------------------------------------------------------------------------------
void CGui::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CGui::InitCamButtons(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data; 

	irr::core::rect<irr::s32> Pos;
	irr::gui::IGUIElement *pE = gl_Data.pGUIEnv->getRootGUIElement();
	irr::video::ITexture *pT;

	// ----------------------------------
	
	Pos = irr::core::rect<irr::s32>(gl_Data.ScreenXY.Width/2-10-36-5-36-5-18,gl_Data.ScreenXY.Height-10-34-10,
									gl_Data.ScreenXY.Width/2-10-36-5-18-5,gl_Data.ScreenXY.Height-10-24);
	
	m_pCamButton[0] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_CAMLEFT,L"");
	pT = gl_Data.pVideoDriver->getTexture("./data/CM_cam_left.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(1,5));
	m_pCamButton[0]->setUseAlphaChannel(true);
	m_pCamButton[0]->setImage(pT);  
	
	Pos.UpperLeftCorner.X  += (5+36+5); 
	Pos.LowerRightCorner.X += (5+36+5);
	
	m_pCamButton[1] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_CAMRIGHT,L"");
	pT = gl_Data.pVideoDriver->getTexture("./data/CM_cam_right.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(1,5));
	m_pCamButton[1]->setUseAlphaChannel(true);
	m_pCamButton[1]->setImage(pT);  

	// ----------------------------------

	Pos = irr::core::rect<irr::s32>(gl_Data.ScreenXY.Width/2-10-36-5-36,gl_Data.ScreenXY.Height-10-68,
									gl_Data.ScreenXY.Width/2-10-36-5,gl_Data.ScreenXY.Height-10-48);
	
	m_pCamButton[2] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_CAMUP,L"");
	pT = gl_Data.pVideoDriver->getTexture("./data/CM_cam_up.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(1,5));
	m_pCamButton[2]->setUseAlphaChannel(true);
	m_pCamButton[2]->setImage(pT);  

	Pos.UpperLeftCorner.Y  = gl_Data.ScreenXY.Height-10-20;
	Pos.LowerRightCorner.Y = gl_Data.ScreenXY.Height-10;

	m_pCamButton[3] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_CAMDOWN,L"");
	pT = gl_Data.pVideoDriver->getTexture("./data/CM_cam_down.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(1,5));
	m_pCamButton[3]->setUseAlphaChannel(true);
	m_pCamButton[3]->setImage(pT);

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CGui::InitGameButtons(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data; 

	irr::core::rect<irr::s32> Pos;
	irr::gui::IGUIElement *pE = gl_Data.pGUIEnv->getRootGUIElement();

	// ----------------------------------
	
	Pos = irr::core::rect<irr::s32>(gl_Data.ScreenXY.Width-20-120, 50,
									gl_Data.ScreenXY.Width-20,     70);

	m_pGameButton[0] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_GAMEAUTO,L"Autogame");

	// ----------------------------------

	Pos.UpperLeftCorner.Y  += 25;
	Pos.LowerRightCorner.Y += 25;

	m_pGameButton[1] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_GAMEQUIT,L"Quit");

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CGui::InitInGameButtons(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data; 

	irr::core::rect<irr::s32> Pos;
	irr::gui::IGUIElement *pE = gl_Data.pGUIEnv->getRootGUIElement();

	// ----------------------------------
	
	irr::video::ITexture *pT = gl_Data.pVideoDriver->getTexture("./data/CM_pause.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(4,4));

	Pos = irr::core::rect<irr::s32>(gl_Data.ScreenXY.Width/2+10,    gl_Data.ScreenXY.Height-10-68,
									gl_Data.ScreenXY.Width/2+10+68, gl_Data.ScreenXY.Height-10);

	m_pInGameButton[0] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_INGAMEPAUSE,L"");
	m_pInGameButton[0]->setUseAlphaChannel(true);

	m_pInGameButton[0]->setImage(pT);  

	// ----------------------------------

	pT = gl_Data.pVideoDriver->getTexture("./data/CM_abort.bmp");
	if (!pT) return(false);
	gl_Data.pVideoDriver->makeColorKeyTexture(pT,irr::core::position2d<irr::s32>(4,4));

	Pos.UpperLeftCorner.X += (68+5); 
	Pos.LowerRightCorner.X += (68+5);

	m_pInGameButton[1] = gl_Data.pGUIEnv->addButton(Pos,pE,ID_BUTTON_INGAMEABORT,L"");
	m_pInGameButton[1]->setUseAlphaChannel(true);

	m_pInGameButton[1]->setImage(pT); 

	// ----------------------------------

	return(true);
}

// ------------------------------------------------------------------------------------------------
void CGui::ToggleF1Window(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	irr::core::stringw sHelptext = L"";

	if (m_pHelpTextBox) // Deleting the object if it shall not be shown anymore
	{
		SAFEDELETE(m_pHelpTextBox);
	}
	else
    {
		irr::core::rect<irr::s32>  rDim( (irr::s32)(gl_Data.ScreenXY.Width  * 0.25f),
										 (irr::s32)(gl_Data.ScreenXY.Height * 0.10f),
										 (irr::s32)(gl_Data.ScreenXY.Width  * 0.75f),
										 (irr::s32)(gl_Data.ScreenXY.Height * 0.75f));
		
		m_pHelpTextBox = new irr::gui::CGUIexScrollWindow(gl_Data.pGUIEnv, gl_Data.Gui.GetGuiFont(),rDim);		
		
		// Read in the help text file
		if (!gl_Data.pIrrlichtDevice->getFileSystem()->existFile("./data/CM_helptext.txt")) sHelptext = L"Helpfile not found.";
		else 
		{
			irr::io::IReadFile *pFile;			
			pFile = gl_Data.pIrrlichtDevice->getFileSystem()->createAndOpenFile("./data/CM_helptext.txt");
			
			irr::s32 iSize = pFile->getSize();
						
			char *dummy = new char[iSize+1];
			memset(dummy,0,iSize+1);
			pFile->read(dummy,iSize);

			for (irr::s32 i=0;i<(iSize+1);i++)
			{
				// Filtering /r, just leaving /n in place
				if (dummy[i] != '\r') sHelptext.append(dummy[i]);
			}
			
			delete[] dummy;
			pFile->drop();
		}

		m_pHelpTextBox->setText(sHelptext.c_str());
	}
}

// ------------------------------------------------------------------------------------------------
irr::gui::IGUIFont* CGui::GetGuiFont(void)
// ------------------------------------------------------------------------------------------------
{
	return(m_pFont);
}

// ------------------------------------------------------------------------------------------------
void CGui::SetPauseSign(bool bIsActive)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	if (bIsActive)
	{
		if (m_pZzz[0]) return;
		
		irr::scene::IAnimatedMesh *pM = gl_Data.pSceneManager->getMesh("./data/CM_Zzz.x");
		if (!pM) return;
		
		for (int i=0;i<3;i++)
		{
			m_pZzz[i] = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0);
			m_pZzz[i]->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));

			m_pZzz[i]->setMaterialFlag(irr::video::EMF_LIGHTING, true);
			m_pZzz[i]->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
				
			irr::core::vector3df v(i/2.0f+0.5f,i/2.0f+0.5f,i/2.0f+0.5f);
			m_pZzz[i]->setScale(v);

			irr::scene::ISceneNodeAnimator *pA = gl_Data.pSceneManager->createRotationAnimator(irr::core::vector3df(0.0f,0.2f+i*0.1f,0.0f));
			m_pZzz[i]->addAnimator(pA);
		}
		
		m_pZzz[0]->setPosition(irr::core::vector3df(0.0f,0.4f,0.0f));
		m_pZzz[1]->setPosition(irr::core::vector3df(0.0f,1.2f,0.0f));
		m_pZzz[2]->setPosition(irr::core::vector3df(0.0f,2.5f,0.0f));
	}
	else
	{
		for (int i=0;i<3;i++) SAFEREMOVE(m_pZzz[i]);
	}
}

// ------------------------------------------------------------------------------------------------
void CGui::SetRunningGame(bool bIsRunning)
// ------------------------------------------------------------------------------------------------
{
	if (bIsRunning)
	{
		m_pGameButton[0]->setEnabled(false);
		m_pGameButton[1]->setEnabled(false);
		
		m_pInGameButton[0]->setEnabled(true);
		m_pInGameButton[1]->setEnabled(true);
	}
	else
	{
		m_pGameButton[0]->setEnabled(true);
		m_pGameButton[1]->setEnabled(true);

		m_pInGameButton[0]->setEnabled(false);
		m_pInGameButton[1]->setEnabled(false);
	}
}

// ------------------------------------------------------------------------------------------------
void CGui::ShowWinner(irr::s32 iPlayer, bool bShowIt)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	if (!bShowIt)
	{
		SAFEREMOVE(m_pWinWindow);
		return;
	}

	if (m_pWinWindow) return;
	
	irr::core::stringc s;
	switch (iPlayer)
	{
	case 0: s="./data/CM_win1.bmp"; break;
	case 1: s="./data/CM_win2.bmp"; break;
	case 2: s="./data/CM_win3.bmp"; break;
	case 3: s="./data/CM_win4.bmp"; break;
	}
	
	irr::core::rect<irr::s32> rec(gl_Data.ScreenXY.Width/2-512/2,gl_Data.ScreenXY.Height/2-128/2,
								  gl_Data.ScreenXY.Width/2+512/2,gl_Data.ScreenXY.Height/2+128/2);
	m_pWinWindow = gl_Data.pGUIEnv->addWindow(rec, false, L"We HAVE A WINNER !");

	m_pWinWindow->getCloseButton()->setVisible(false);
	m_pInGameButton[0]->setEnabled(false);
	
	irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture(s.c_str());
	if (!pT) return;
	
	irr::core::position2d<irr::s32> pos(0,0);
	gl_Data.pGUIEnv->addImage(pT, pos, false, m_pWinWindow);
}

// ------------------------------------------------------------------------------------------------
bool CGui::InitDemoSign(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	irr::gui::IGUIStaticText* pT = gl_Data.pGUIEnv->addStaticText(L"Chase-Me: Irrlicht Demo Game. Press F1 for help, D for debug info.",
																  irr::core::rect<irr::s32>(0,0,512,50));
	pT->setOverrideColor(irr::video::SColor(200,200,200,0));
	return(true);
}
