// map.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CMap::CMap(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
CMap::~CMap(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CMap::Loop(void)
// ------------------------------------------------------------------------------------------------
{
	ProcessMoveJobs();
}

// ------------------------------------------------------------------------------------------------
bool CMap::Init(void)
// ------------------------------------------------------------------------------------------------
{
	if (!InitSkyBox())			  return(false); 
	if (!InitGamePlane())		  return(false); 
	if (!InitMainWalkingFields()) return(false); 
	if (!InitStartFields())       return(false); 
	if (!InitHomeFields())        return(false); 
	if (!InitInitFields())		  return(false); 
	if (!InitPlayerTokens())	  return(false); 

	Reset();

	return(true);
}

// ------------------------------------------------------------------------------------------------
void CMap::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CMap::Reset(void)
// ------------------------------------------------------------------------------------------------
{
	DeleteMoveJobs();
	SetAllTokensToStartFields();
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitSkyBox(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	irr::video::ITexture* T[6];
	
	T[0] = gl_Data.pVideoDriver->getTexture("./data/CM_top.bmp");
	if (!T[0]) return(false);
	T[1] = gl_Data.pVideoDriver->getTexture("./data/CM_bottom.bmp");
	if (!T[1]) return(false);
	T[2] = gl_Data.pVideoDriver->getTexture("./data/CM_left.bmp"); 
	if (!T[2]) return(false);
	T[3] = gl_Data.pVideoDriver->getTexture("./data/CM_right.bmp"); 
	if (!T[3]) return(false);
	T[4] = gl_Data.pVideoDriver->getTexture("./data/CM_front.bmp"); 
	if (!T[4]) return(false);
	T[5] = gl_Data.pVideoDriver->getTexture("./data/CM_back.bmp"); 
	if (!T[5]) return(false);
	
	gl_Data.pSceneManager->addSkyBoxSceneNode(T[0],T[1],T[2],T[3],T[4],T[5],gl_Data.pSceneManager->getRootSceneNode(),0); 

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitGamePlane(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_Plane.x");
	if (!pM) return(false);

	m_pPlaneNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0);
	
	// Add triangle selector for collision detection with the dice later on
	irr::scene::ITriangleSelector* s = gl_Data.pSceneManager->createTriangleSelector(pM->getMesh(0), m_pPlaneNode);
	m_pPlaneNode->setTriangleSelector(s);
	s->drop();

	m_pPlaneNode->setScale(irr::core::vector3df(3.0f,1.0f,3.0f));
	
	m_pPlaneNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
	m_pPlaneNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true);

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitMainWalkingFields(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	for (irr::s32 i=0;i<5;i++) m_MainFields[i].vPosition = irr::core::vector3df(-1.0f, 0.0f, (-5.0f+i));
	for (irr::s32 i=0;i<4;i++) m_MainFields[5+i].vPosition = irr::core::vector3df((-2.0f-i), 0.0f, -1.0f);
	
	m_MainFields[9].vPosition = irr::core::vector3df(-5.0f, 0.0f, 0.0f);
	
	for (irr::s32 i=0;i<5;i++) m_MainFields[10+i].vPosition = irr::core::vector3df((-5.0f+i), 0.0f, +1.0f);
	for (irr::s32 i=0;i<4;i++) m_MainFields[15+i].vPosition = irr::core::vector3df(-1.0f, 0.0f, (2.0f+i));
	
	m_MainFields[19].vPosition = irr::core::vector3df(0.0f, 0.0f, 5.0f);
	
	for (irr::s32 i=0;i<5;i++) m_MainFields[20+i].vPosition = irr::core::vector3df(1.0f, 0.0f, (5.0f-i));
	for (irr::s32 i=0;i<4;i++) m_MainFields[25+i].vPosition = irr::core::vector3df((2.0f+i), 0.0f, +1.0f);
	
	m_MainFields[29].vPosition = irr::core::vector3df(5.0f, 0.0f, 0.0f);
	
	for (irr::s32 i=0;i<5;i++) m_MainFields[30+i].vPosition = irr::core::vector3df((5.0f-i), 0.0f, -1.0f);
	for (irr::s32 i=0;i<4;i++) m_MainFields[35+i].vPosition = irr::core::vector3df(1.0f, 0.0f, (-2.0f-i));
	
	m_MainFields[39].vPosition = irr::core::vector3df(0.0f, 0.0f, -5.0f);

	for (irr::s32 i=0;i<40;i++)
	{
		irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_NormalField.x");
		if (!pM) return(false);

		m_MainFields[i].pNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0,irr::core::vector3df(20.0f,0.0f,0.0f));
		
		m_MainFields[i].pNode->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));
		m_MainFields[i].pNode->setPosition(m_MainFields[i].vPosition);
		
		m_MainFields[i].pNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
		m_MainFields[i].pNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
	}

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitStartFields(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	for (irr::s32 i=0;i<2;i++) 
	{
		m_StartFields[0][  i].vPosition = irr::core::vector3df((-5.0f+i), 0.0f, -4.0f);
		m_StartFields[0][2+i].vPosition = irr::core::vector3df((-5.0f+i), 0.0f, -5.0f);
		m_StartFields[1][  i].vPosition = irr::core::vector3df((-5.0f+i), 0.0f,  5.0f);
		m_StartFields[1][2+i].vPosition = irr::core::vector3df((-5.0f+i), 0.0f,  4.0f);
		m_StartFields[2][  i].vPosition = irr::core::vector3df(( 4.0f+i), 0.0f,  5.0f);
		m_StartFields[2][2+i].vPosition = irr::core::vector3df(( 4.0f+i), 0.0f,  4.0f);
		m_StartFields[3][  i].vPosition = irr::core::vector3df(( 4.0f+i), 0.0f, -4.0f);
		m_StartFields[3][2+i].vPosition = irr::core::vector3df(( 4.0f+i), 0.0f, -5.0f);
	}

	for (irr::s32 i=0;i<4;i++) // Players
	for (irr::s32 ii=0;ii<4;ii++) // Fields per player
	{
		irr::core::stringc s;
		switch (i)
		{
		case 0: s = "./data/CM_StartField1.bmp"; break; 
		case 1: s = "./data/CM_StartField2.bmp"; break; 
		case 2: s = "./data/CM_StartField3.bmp"; break; 
		case 3: s = "./data/CM_StartField4.bmp"; break; 
		}

		irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_NormalField.x");
		if (!pM) return(false);

		m_StartFields[i][ii].pNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0,irr::core::vector3df(20.0f,0.0f,0.0f));
		
		// Assign other texture, as standard is "CM_NormalField1.bmp"
		irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture(s.c_str());
		if (!pT) return(false);

		m_StartFields[i][ii].pNode->setMaterialTexture(0, pT);

		m_StartFields[i][ii].pNode->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));
		m_StartFields[i][ii].pNode->setPosition(m_StartFields[i][ii].vPosition);
		
		m_StartFields[i][ii].pNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
		m_StartFields[i][ii].pNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
	}

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitHomeFields(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	for (irr::s32 i=0;i<4;i++) 
	{
		m_EndFields[0][i].vPosition = irr::core::vector3df(               0, 0, (irr::f32)(-4+i));
		m_EndFields[1][i].vPosition = irr::core::vector3df((irr::f32)(-4+i), 0,                0);
		m_EndFields[2][i].vPosition = irr::core::vector3df(               0, 0, (irr::f32)( 4-i));
		m_EndFields[3][i].vPosition = irr::core::vector3df((irr::f32)( 4-i), 0,                0);
	}

	for (irr::s32 i=0;i<4;i++) // Players
	for (irr::s32 ii=0;ii<4;ii++) // Fields per player
	{
		irr::core::stringc s;
		switch (i)
		{
		case 0: s = "./data/CM_StartField1.bmp"; break; 
		case 1: s = "./data/CM_StartField2.bmp"; break; 
		case 2: s = "./data/CM_StartField3.bmp"; break; 
		case 3: s = "./data/CM_StartField4.bmp"; break; 
		}
		
		irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_HomeField.x");
		if (!pM) return(false);

		m_EndFields[i][ii].pNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0,irr::core::vector3df(20.0f,0.0f,0.0f));
		
		// Assign other texture, as standard is "CM_NormalField1.bmp"
		irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture(s.c_str());
		if (!pT) return(false);

		m_EndFields[i][ii].pNode->setMaterialTexture(0, pT);

		m_EndFields[i][ii].pNode->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));
		m_EndFields[i][ii].pNode->setPosition(m_EndFields[i][ii].vPosition);
		
		m_EndFields[i][ii].pNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
		m_EndFields[i][ii].pNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
	}

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitInitFields(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	for (irr::s32 i=0;i<4;i++) // Players
	{
		irr::core::stringc s;
		switch (i)
		{
		case 0: s = "./data/CM_InitField1.bmp"; break; 
		case 1: s = "./data/CM_InitField2.bmp"; break; 
		case 2: s = "./data/CM_InitField3.bmp"; break; 
		case 3: s = "./data/CM_InitField4.bmp"; break; 
		}
	
		irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture(s.c_str());
		if (!pT) return(false);
		m_MainFields[10*i].pNode->setMaterialTexture(0, pT);
	}

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::InitPlayerTokens(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	for (irr::s32 i=0;i<4;i++) // Players
	for (irr::s32 ii=0;ii<4;ii++) // Tokens per player
	{
		irr::core::stringc s;
		switch (i)
		{
		case 0: s = "./data/CM_PlayerToken1.bmp"; break; 
		case 1: s = "./data/CM_PlayerToken2.bmp"; break; 
		case 2: s = "./data/CM_PlayerToken3.bmp"; break; 
		case 3: s = "./data/CM_PlayerToken4.bmp"; break; 
		}

		irr::scene::IAnimatedMesh* pM = gl_Data.pSceneManager->getMesh("./data/CM_PlayerToken.x");
		if (!pM) return(false);

		m_Tokens[i][ii].pNode = gl_Data.pSceneManager->addAnimatedMeshSceneNode(pM, gl_Data.pSceneManager->getRootSceneNode(),0,irr::core::vector3df(20.0f,0.0f,0.0f));
		
		// Assign other texture, as standard is "CM_NormalField1.bmp"
		irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture(s.c_str());
		if (!pT) return(false);

		m_Tokens[i][ii].pNode->setMaterialTexture(0, pT);
		
		m_Tokens[i][ii].pNode->setScale(irr::core::vector3df(0.2f,0.2f,0.2f));
		m_Tokens[i][ii].pNode->setPosition(irr::core::vector3df(0.0f,0.0f,0.0f));
		
		m_Tokens[i][ii].pNode->setMaterialFlag(irr::video::EMF_LIGHTING, true);
		m_Tokens[i][ii].pNode->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
	}

	SetAllTokensToStartFields();

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CMap::IsATokenOnTheMove(void)
// ------------------------------------------------------------------------------------------------
{
	// If there are movejobs pending, return true
	if (m_aMoveJobs.empty()) return(false);

	return(true);
}

// ------------------------------------------------------------------------------------------------
void CMap::SetAllTokensToStartFields(void)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++) // Players
	for (irr::s32 ii=0;ii<4;ii++) // Tokens per player
	{
		// Assigning index -4..-1 to 0..3 (pos index)
		irr::core::vector3df vPos = m_StartFields[i][ii].vPosition;
		vPos.Y += 0.2f;
		m_Tokens[i][ii].pNode->removeAnimators();
		m_Tokens[i][ii].pNode->setPosition(vPos);
		m_Tokens[i][ii].iRelativeIndex = -ii-1;
		m_Tokens[i][ii].iAbsoluteIndex = GetAbsoluteFromRelativeIndex(i,m_Tokens[i][ii].iRelativeIndex);
	}
}

// RelIndex is -4..-1 (for init position), 0..39 (normal) and 40..43 (for home position)
// AbsIndex is -4..-1 (for init position), 0..39 (normal) and 100..103 (for home position)
// ------------------------------------------------------------------------------------------------
irr::s32 CMap::GetAbsoluteFromRelativeIndex(irr::s32 iPlayer, irr::s32 iRelIndex)
// ------------------------------------------------------------------------------------------------
{
	if (iRelIndex < 0) return(iRelIndex); // is still in init position
	
	if (iRelIndex > 39) return(100 + iRelIndex - 40); // home position 100..103
	
	irr::s32 iAbsPos = iRelIndex + (iPlayer * 10);
	if (iAbsPos > 39) iAbsPos -= 40;
	
	return(iAbsPos);
}

// RelIndex is -4..-1 (for init position), 0..39 (normal) and 40..43 (for home position)
// AbsIndex is -4..-1 (for init position), 0..39 (normal) and 100..103 (for home position)
// ------------------------------------------------------------------------------------------------
irr::s32 CMap::GetRelativeFromAbsoluteIndex(irr::s32 iPlayer, irr::s32 iAbsIndex)
// ------------------------------------------------------------------------------------------------
{
	if (iAbsIndex < 0) return(iAbsIndex); // is still in init position

	if (iAbsIndex > 99) return(40 + iAbsIndex - 100); // home position 100..103 is 40..43
	
	irr::s32 iRelPos = iAbsIndex - (iPlayer * 10);
	if (iRelPos < 0) iRelPos += 40;
	
	return(iRelPos);
}

// ------------------------------------------------------------------------------------------------
void CMap::EnableActivePlayerSign(irr::s32 iPlayer)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	for (irr::s32 i=0;i<4;i++)
	{
		irr::scene::IBillboardSceneNode* pN = gl_Data.pSceneManager->addBillboardSceneNode(
																m_Tokens[iPlayer][i].pNode,
																irr::core::dimension2d<irr::f32>(1.0f,1.0f),
																irr::core::vector3df(0.0f,3.0f,0.0f),
																0);

		irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture("./data/CM_hilight.bmp");
		if (!pT) return;

		pN->setMaterialType(irr::video::EMT_TRANSPARENT_ADD_COLOR ); 
	
		pN->setMaterialTexture(0,pT);
		pN->setMaterialFlag(irr::video::EMF_LIGHTING, true);
		pN->setMaterialFlag(irr::video::EMF_NORMALIZE_NORMALS, true); 
	}
}

// ------------------------------------------------------------------------------------------------
void CMap::DisableActivePlayerSign(irr::s32 iPlayer)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++)
	{
		m_Tokens[iPlayer][i].pNode->removeAll();
	}
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerGotItemAtRel0(irr::s32 iPlayer, irr::s32& iRetValue)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++)
	{
		if (m_Tokens[iPlayer][i].iRelativeIndex == 0)
		{
			iRetValue = i;
			return(true);
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerGotOutOfGameItems(irr::s32 iPlayer, irr::s32& iRetValue)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++)
	{
		if (m_Tokens[iPlayer][i].iRelativeIndex < 0)
		{
			iRetValue = i;
			return(true);
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerGotIngameItems(irr::s32 iPlayer, irr::s32& iRetValue)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++)
	{
		if (m_Tokens[iPlayer][i].iRelativeIndex > -1)
		{
			iRetValue = i;
			return(true);
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerItemCanWalk(irr::s32 iPlayer, irr::s32 iItem, irr::s32 iDiceRoll)
// ------------------------------------------------------------------------------------------------
{
	// Is it out of game?
	if (m_Tokens[iPlayer][iItem].iRelativeIndex < 0)
	{
		if (iDiceRoll != 6) return(false);
		for (irr::s32 i=0;i<4;i++)
		{
			// We could move out, but have another own token there
			if (m_Tokens[iPlayer][i].iRelativeIndex == 0) return(false);
		}
		return(true); // OK, can move in
	}
	
	// Would we be over the final boundary?
	irr::s32 iRelTargetIndex = m_Tokens[iPlayer][iItem].iRelativeIndex + iDiceRoll;
	if (iRelTargetIndex > 43) return(false);

	// Is there another of my items already? 
	for (irr::s32 i=0;i<4;i++)
	{
		if (m_Tokens[iPlayer][i].iRelativeIndex == iRelTargetIndex) return(false); 
	}
	
	return(true); // Seems this one could move. Not checking if slaying another player's item
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerCanSlayAnother(irr::s32 iPlayer, irr::s32 iDiceRoll, irr::s32& iRetItem)
// ------------------------------------------------------------------------------------------------
{
	// NOT checking: Bringing out-of-game tokens in

	for (irr::s32 i=0;i<4;i++) // moving player's tokens
	{
		if (CheckIfPlayerItemCanWalk(iPlayer, i, iDiceRoll))
		{
			irr::s32 iTargetAbs = GetAbsoluteFromRelativeIndex(iPlayer, m_Tokens[iPlayer][i].iRelativeIndex+iDiceRoll);
		
			for (irr::s32 a=0;a<4;a++) // other players
			{
				if (a != iPlayer) // not checking the moving player 
                    for (irr::s32 b=0;b<4;b++) // tokens of the ther players
                        if (m_Tokens[a][b].iAbsoluteIndex == iTargetAbs) // Other player's token there!
						{
							iRetItem = i;
							return(true); 
						}
			}
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerCanBringItemHome(irr::s32 iPlayer, irr::s32 iDiceRoll, irr::s32& iRetItem)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++) // items
	{
		if (CheckIfPlayerItemCanWalk(iPlayer, i, iDiceRoll))
		{
			if ((m_Tokens[iPlayer][i].iRelativeIndex + iDiceRoll) > 39)
			{
				iRetItem = i;
				return(true);
			}
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfaPlayerItemCanWalk(irr::s32 iPlayer, irr::s32 iDiceRoll, irr::s32& iRetItem)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++) // items
	{
		if (CheckIfPlayerItemCanWalk(iPlayer, i, iDiceRoll))
		{
			iRetItem = i;
			return(true);
		}
	}

	return(false);
}

// AbsIndex is -4..-1 (for init position), 0..39 (normal) and 100..103 (for home position)
// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfItemIsAtAbsoluteIndex(irr::s32 iCallingPlayer, irr::s32 iAbsIndex, irr::s32& iPlayer, irr::s32& iRetItem)
// ------------------------------------------------------------------------------------------------
{
	// iCallingPlayer is needed for checking home/start fields. pass -1 to not use
	if ((iCallingPlayer != -1)&&(iAbsIndex > 99)) // Home fields
	{
		for (irr::s32 ii=0;ii<4;ii++) // items
		{
			if (m_Tokens[iCallingPlayer][ii].iAbsoluteIndex == iAbsIndex)
			{
				iPlayer  = iCallingPlayer;
				iRetItem = ii;
				return(true); 
			}
		}
		return(false); 
	}
	else if ((iCallingPlayer != -1)&&(iAbsIndex < 0)) // Start fields
	{
		for (irr::s32 ii=0;ii<4;ii++) // items
		{
			if (m_Tokens[iCallingPlayer][ii].iAbsoluteIndex == iAbsIndex)
			{
				iPlayer  = iCallingPlayer;
				iRetItem = ii;
				return(true); 
			}
		}
		return(false); 
	}

	if ((iAbsIndex > 99)||(iAbsIndex < 0)) return(false); // can't check

	// Checking normal walking fields
	for (irr::s32 i=0;i<4;i++) // players
	{
		for (irr::s32 ii=0;ii<4;ii++) // items
		{
			if ((m_Tokens[i][ii].iAbsoluteIndex == iAbsIndex))
			{
				iPlayer = i;
				iRetItem = ii;
				return(true);   
			}
		}
	}

	return(false);
}

// ------------------------------------------------------------------------------------------------
void CMap::MovePlayerItem(irr::s32 iPlayer, irr::s32 iItem, irr::s32 iDiceRoll)
// ------------------------------------------------------------------------------------------------
{
	irr::s32 newRelIndex;
	irr::s32 newAbsIndex;
	
	// Special: Item is out of game (and dice = 6)
	if (m_Tokens[iPlayer][iItem].iRelativeIndex < 0) newRelIndex = 0;
	else newRelIndex = m_Tokens[iPlayer][iItem].iRelativeIndex + iDiceRoll;
  
	newAbsIndex = GetAbsoluteFromRelativeIndex(iPlayer, newRelIndex);
	
	InitMoveJob(iPlayer,iItem,newRelIndex);
}

// ------------------------------------------------------------------------------------------------
void CMap::InitMoveJob(irr::s32 iPlayer, irr::s32 iItem, irr::s32 newRelIndex)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
		
	if (newRelIndex > 39) // Going to homezone / move within homezone
	{
		if (m_Tokens[iPlayer][iItem].iRelativeIndex < 39) // from outside
		{
			for (irr::s32 i=m_Tokens[iPlayer][iItem].iRelativeIndex+1; i<40; i++)
			{
				irr::s32 iNewAbsIndex = GetAbsoluteFromRelativeIndex(iPlayer, i);
			
				MOVEJOB m;
				m.vEndPosition = m_MainFields[iNewAbsIndex].vPosition;
				
				// If there is another item, jump over it
				irr::s32 s1,s2;
				if (CheckIfItemIsAtAbsoluteIndex(iPlayer, iNewAbsIndex, s1,s2)) m.vEndPosition.Y += 1.2f;
				else m.vEndPosition.Y += 0.2f;

				m.pNode = m_Tokens[iPlayer][iItem].pNode;
			
				m_aMoveJobs.push_back(m);
				m_Tokens[iPlayer][iItem].iAbsoluteIndex = iNewAbsIndex;
				m_Tokens[iPlayer][iItem].iRelativeIndex = i;
			}
		}
		
		// Now, we are one step before homezone or within
		for (irr::s32 i=m_Tokens[iPlayer][iItem].iRelativeIndex+1; i<(newRelIndex+1); i++)
		{
			irr::s32 iNewAbsIndex = GetAbsoluteFromRelativeIndex(iPlayer, i);
			
			MOVEJOB m;
			m.vEndPosition = m_EndFields[iPlayer][i-40].vPosition;

			// If there is another item, jump over it
			irr::s32 s1,s2;
			if (CheckIfItemIsAtAbsoluteIndex(iPlayer, iNewAbsIndex, s1,s2)) m.vEndPosition.Y += 1.2f;
			else m.vEndPosition.Y += 0.1f;
			
			m.pNode = m_Tokens[iPlayer][iItem].pNode;
		
			m_aMoveJobs.push_back(m);
			m_Tokens[iPlayer][iItem].iAbsoluteIndex = iNewAbsIndex;
			m_Tokens[iPlayer][iItem].iRelativeIndex = i;

		}
	}
	else if (newRelIndex == 0) // Going to init field
	{
		irr::s32 iNewAbsIndex = GetAbsoluteFromRelativeIndex(iPlayer, 0);

		MOVEJOB m;
		m.vEndPosition = m_MainFields[iNewAbsIndex].vPosition;
		m.vEndPosition.Y += 0.2f;
		
		// Slay, if something on target field
		CheckForSlayingItem(iNewAbsIndex);
		
		m.pNode = m_Tokens[iPlayer][iItem].pNode;
			
		m_aMoveJobs.push_back(m);
		m_Tokens[iPlayer][iItem].iAbsoluteIndex = iNewAbsIndex;
		m_Tokens[iPlayer][iItem].iRelativeIndex = 0;
	}
	else // walking normal
	{
		for (irr::s32 i=m_Tokens[iPlayer][iItem].iRelativeIndex+1; i<(newRelIndex+1); i++)
		{
			irr::s32 iNewAbsIndex = GetAbsoluteFromRelativeIndex(iPlayer, i);

			MOVEJOB m;
			m.vEndPosition = m_MainFields[iNewAbsIndex].vPosition;
			
			// Slay, if something on target field (last in walking line)
			if (i==newRelIndex) CheckForSlayingItem(iNewAbsIndex);
			
			// If there is another item, jump over it
			irr::s32 s1,s2;
			if (CheckIfItemIsAtAbsoluteIndex(iPlayer, iNewAbsIndex, s1,s2)) m.vEndPosition.Y += 1.2f;
			else m.vEndPosition.Y += 0.2f;

			m.pNode = m_Tokens[iPlayer][iItem].pNode;
			
			m_aMoveJobs.push_back(m);
			m_Tokens[iPlayer][iItem].iAbsoluteIndex = iNewAbsIndex;
			m_Tokens[iPlayer][iItem].iRelativeIndex = i;
		}
	}
}

// ------------------------------------------------------------------------------------------------
void CMap::CheckForSlayingItem(irr::s32 iAbsIndex)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	// Check if there is a token at target field. Send token home, if so

	irr::s32 iP, iI;
	if (CheckIfItemIsAtAbsoluteIndex(-1, iAbsIndex, iP, iI)) // something to slay there!
	{
		irr::s32 iNewRelIndex = -iI-1; // Start field Index

		MOVEJOB m;
		m.vEndPosition = m_StartFields[iP][iI].vPosition; // Item 0 = startfield 0 etc.
		m.vEndPosition.Y += 0.2f;

		m.pNode = m_Tokens[iP][iI].pNode;
			
		m_aMoveJobs.push_back(m);
		m_Tokens[iP][iI].iRelativeIndex = iNewRelIndex;
		m_Tokens[iP][iI].iAbsoluteIndex = GetAbsoluteFromRelativeIndex(iP,m_Tokens[iP][iI].iRelativeIndex);
	}
}

// ------------------------------------------------------------------------------------------------
bool CMap::CheckIfPlayerHasWon(irr::s32 iPlayer)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4;i++) // items
	{
		if (m_Tokens[iPlayer][i].iAbsoluteIndex < 100) return(false);
	}

	return(true);
}

// ------------------------------------------------------------------------------------------------
void CMap::ProcessMoveJobs(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	static bool bRunning = false;
	
	if (m_aMoveJobs.empty())
	{
		bRunning = false;
		return;
	}	

	if (!bRunning) // initialize pending job
	{
		irr::scene::ISceneNodeAnimator* pA=gl_Data.pSceneManager->createFlyStraightAnimator(
																	m_aMoveJobs[0].pNode->getPosition(),
																	m_aMoveJobs[0].vEndPosition,500,false);
		m_aMoveJobs[0].pNode->removeAnimators();
		m_aMoveJobs[0].pNode->addAnimator(pA);
		pA->drop();
		bRunning = true;
	}
	else // check if finished. Remove, if so.
	{
		if (m_aMoveJobs[0].vEndPosition == m_aMoveJobs[0].pNode->getAbsolutePosition())
		{
			m_aMoveJobs.erase(0);
			bRunning = false;
		}
	}
}

// ------------------------------------------------------------------------------------------------
void CMap::DeleteMoveJobs(void)
// ------------------------------------------------------------------------------------------------
{
	m_aMoveJobs.clear();
}
