// player.cpp
#include "global.h"

// ------------------------------------------------------------------------------------------------
CPlayer::CPlayer(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
CPlayer::~CPlayer(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Loop(void)
// ------------------------------------------------------------------------------------------------
{
	switch (m_State)
	{
	case PS_NONE:   if (MakeActive())   m_State=PS_ACTIVE; break;
	case PS_ACTIVE: if (Wait1())	    m_State=PS_WAIT1;  break;
	case PS_WAIT1:  if (Roll())         m_State=PS_ROLL;   break;
	case PS_ROLL:   if (Move())         m_State=PS_MOVE;   break;
	case PS_MOVE:   if (Wait2())        m_State=PS_WAIT2;  break;
	case PS_WAIT2:  if (MakeInactive()) m_State=PS_NONE;   return(true); break;
	}	
	
	return(false); // Not finished
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Init(irr::s32 i)
// ------------------------------------------------------------------------------------------------
{
	
	m_PlayerNumber = i;
	Reset();
	
	return(true);
}

// ------------------------------------------------------------------------------------------------
void CPlayer::Exit(void)
// ------------------------------------------------------------------------------------------------
{
}

// ------------------------------------------------------------------------------------------------
void CPlayer::Reset(void)
// ------------------------------------------------------------------------------------------------
{
	m_State = PS_NONE;
	MakeInactive();
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::MakeActive(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	gl_Data.Map.EnableActivePlayerSign(m_PlayerNumber);

	return(true); // finished
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Wait1(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// Check if this is a an autogame. Request diceroll then
	if (gl_Data.GameState == autogame)
	{
		m_DiceResult = gl_Data.Dice.Roll(m_PlayerNumber);
		return(true);
	}

	return(false); // not finished
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Roll(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// Wait until dice animation has finished. Then, wait another sec
	static irr::s32 iWait;

	if (gl_Data.Dice.IsDiceOnTheMove())
	{
		iWait = gl_Data.pVideoDriver->getFPS();
		return(false);
	}
	else
	{
		iWait -= 1;
		if (iWait < 0) return(true);
		else return(false);
	}
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Move(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// Calculate movement of this dice roll. Check for manual game laters
	if (gl_Data.GameState != autogame) return(true);

	irr::s32 dummy1, dummy2;

	// ---------------------------------------------------------------------------------------
	// Check if there is an own item at start position, can move and others are still homed.
	// If so, handle this one first

	if ( gl_Data.Map.CheckIfPlayerGotItemAtRel0(m_PlayerNumber,dummy1) &&
		 gl_Data.Map.CheckIfPlayerGotOutOfGameItems(m_PlayerNumber,dummy2))
	{
		if (gl_Data.Map.CheckIfPlayerItemCanWalk(m_PlayerNumber,dummy1,m_DiceResult))
		{
			gl_Data.Map.MovePlayerItem(m_PlayerNumber, dummy1, m_DiceResult);
			return(true);
		} 
	}

	// ---------------------------------------------------------------------------------------
	// Check if out-of-game items there and dice = 6. Must move out then (if possible)

	if (gl_Data.Map.CheckIfPlayerGotOutOfGameItems(m_PlayerNumber, dummy1) &&
		gl_Data.Map.CheckIfPlayerItemCanWalk(m_PlayerNumber, dummy1, m_DiceResult))
	{
		gl_Data.Map.MovePlayerItem(m_PlayerNumber, dummy1, m_DiceResult);
		return(true);
	}

	// ---------------------------------------------------------------------------------------
	// Check if I can hit another player. Do so, if possible	

	if (gl_Data.Map.CheckIfPlayerCanSlayAnother(m_PlayerNumber, m_DiceResult, dummy1))
	{
		gl_Data.Map.MovePlayerItem(m_PlayerNumber, dummy1, m_DiceResult);
		return(true);
	}

	
	// ---------------------------------------------------------------------------------------
	// Check if I can bring an item into home position
    if (gl_Data.Map.CheckIfPlayerCanBringItemHome(m_PlayerNumber, m_DiceResult, dummy1))
	{
		gl_Data.Map.MovePlayerItem(m_PlayerNumber, dummy1, m_DiceResult);
		return(true);
	}
	
	// ---------------------------------------------------------------------------------------
	// Check if I can walk normal. This does cover movement within home zone
	if (gl_Data.Map.CheckIfaPlayerItemCanWalk(m_PlayerNumber, m_DiceResult, dummy1))
	{
		gl_Data.Map.MovePlayerItem(m_PlayerNumber, dummy1, m_DiceResult);
		return(true);
	}

	// If here, we cannot move. Show info, maybe
	return(true);
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::Wait2(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;

	// Wait until token animation has finished. Then, wait another sec
	static irr::s32 iWait;

	if (gl_Data.Map.IsATokenOnTheMove())
	{
		iWait = gl_Data.pVideoDriver->getFPS();
		return(false);
	}
	else
	{
		iWait -= 1;
		if (iWait < 0) return(true);
		else return(false);
	}
}

// ------------------------------------------------------------------------------------------------
bool CPlayer::MakeInactive(void)
// ------------------------------------------------------------------------------------------------
{
	extern GLOBALDATA gl_Data;
	
	gl_Data.Map.DisableActivePlayerSign(m_PlayerNumber);

	return(true); // finished
}
